#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, sys, os
import tempfile
import base64

import gobject, gtk, pango
from gtk import TRUE, FALSE
from gtk import gdk

from detectcharset import detectcharset


skins_dir = 'skins'

#PANGO_SCALE = pango.SCALE
PANGO_SCALE = 1024

FILE_TYPE_PLAIN_TEXT = 0
FILE_TYPE_FB2 = 1

TYPE_TEXT = 0
TYPE_TITLE = 1
TYPE_NEW_LINE = 2
TYPE_IMAGE = 3

class PangoLayoutWord:
    pangolayout = None
    width = 0
    index = 0
    style = None


class TextLine:
    type = None
    text = ''
    image = None
    style = None


class TextParagraph:
    offset = 0
    lines = []




class OrnamentBook(gtk.DrawingArea):

    keysyms_next = (gtk.keysyms.KP_Page_Down,
                    gtk.keysyms.Page_Down,
                    gtk.keysyms.space,
                    gtk.keysyms.Right)
    keysyms_prev = (gtk.keysyms.KP_Page_Up,
                    gtk.keysyms.Page_Up,
                    gtk.keysyms.Left)

    def __init__(self, parent):


        default_skin = 'golden.jpg'
        skin_file = None
        for d in sys.path:
            f = os.path.join(d, 'ornamentbook', skins_dir, default_skin)
            if os.path.exists(f):
                skin_file = f
                break
        if not skin_file:
            sys.exit('ERROR: can\'t find default skin file %s' % default_skin)
        self.skin_file = skin_file

        gtk.DrawingArea.__init__(self)

        self._parent = parent
        #self.area = gtk.DrawingArea()

##         self.set_size_request(950, 400)
##         self._width = 950
##         self._height = 400

        gobject.signal_new('position-changed', OrnamentBook,
                           gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE,
                           (gobject.TYPE_FLOAT,))

        self.set_size_request(600, 400)
        self._width = 600
        self._height = 400

        #self.set_events(gdk.ALL_EVENTS_MASK)
        #self.add_events(gdk.BUTTON_PRESS_MASK|gdk.BUTTON_RELEASE_MASK)
        #self.add_events(gdk.BUTTON_RELEASE_MASK)
        self.add_events(gdk.BUTTON_PRESS_MASK)
        #self.add_events(gdk.BUTTON_PRESS_MASK|gdk.BUTTON_RELEASE_MASK|gdk.KEY_PRESS_MASK|gdk.KEY_RELEASE_MASK)
        #self.set_events(gdk.ALL_EVENTS_MASK)

        self.connect('expose-event', self.area_expose_cb)
        #self.connect('button_release_event', self.button_release_event_cb)
        self.connect('button_press_event', self.button_release_event_cb)
        #self.realize()

        #self._parent.connect('key_press_event', self.key_press_event_cb)
        self._parent.connect('key-release-event', self.key_release_event_cb)

        self.text = ''
        self.lines = []

        self.current_page_line_index = 0
        self.current_page_char_index = 0

        self.next_page_line_index = 0
        self.next_page_char_index = 0

        self.pixmap_area = None
        #self.prev_pixmap_area = None
        #self.next_pixmap_area = None

        self.set_font('Normal')
        self.color = gdk.color_parse('#000000')

        self.top_margin = 10
        self.bottom_margin = 10
        self.left_page_left_margin = 30
        self.left_page_right_margin = 30
        self.right_page_left_margin = 30
        self.right_page_right_margin = 30

        #self.tabs_width = 5
        self.set_tab_width(5)

        self.file_type = FILE_TYPE_PLAIN_TEXT

        return


    def set_skin(self, filename):
        try:
            open(filename)
        except IOError, err:
            sys.stderr.write('ERROR: can\'t open file: %s: %s\n'
                             % (filename, err[1]))
            return
        self.skin_file = filename
        self.pixbuf = gdk.pixbuf_new_from_file(self.skin_file)
        self.cur_pixbuf = self.pixbuf.scale_simple(self._width, self._height,
                                                   gdk.INTERP_BILINEAR)



    def open_file(self, filename, encoding=None, line=0, index=0):
        #print '>open_file', line, index

        if filename[-4:] == '.fb2':

            self.open_fb2_file(filename, line, index)

        else:

            if not encoding:
                fd = open(filename)
                enc = detectcharset(fd)
                if enc:
                    self.encoding = enc
                else:
                    self.encoding = 'iso8859-1'
            self.open_plain_text_file(filename, encoding, line, index)

        self.draw_text()


    def open_plain_text_file(self, filename, encoding,
                             line_index=0, char_index=0):

        #print 'open_plain_text_file', line_index, char_index
        #fd = open(filename)
        text = open(filename).read()

        text = unicode(text, encoding)


        self.file_type = FILE_TYPE_PLAIN_TEXT

        self.text = text #unicode(text, 'utf-8')
        lines = self.text.splitlines()
        self.lines = []
        self.styles = {}
        for l in lines:
            self.lines.append((TYPE_TEXT, l))
        self.current_page_line_index = line_index
        self.current_page_char_index = char_index
        #self.pixmap_area = None
        #self.prev_pixmap_area = None
        #self.next_pixmap_area = None

        gtk.idle_add(self.emit_position_changed)




    def open_fb2_file(self, filename, line_index=0, char_index=0):
        from fictionbook2 import FictionBook2
        import tempfile

        #print 'open_fb2_file', line_index, char_index

        self.file_type = FILE_TYPE_FB2

        fb = FictionBook2(filename)
        self._fb = fb
        text = unicode(fb.text, 'utf-8')

        self.text = text
        self.lines = []
        lines = text.splitlines()
        for l in lines:
            self.lines.append([TYPE_TEXT, l])

        #self.len_text = len(self.text)

        #self.styles = fb.styles


        sl = {}
        for style in fb.styles:
            #print style[0], line_index
            if sl.has_key(style[0]+1):
                sl[style[0]+1].append((style[1], style[1]+style[2], style[3]))
            else:
                sl[style[0]+1] = [(style[1], style[1]+style[2], style[3])]
##             if style[0] == line_index-1:
##                 sl.append(style)
##                 #print self.lines[line_index][1].encode('koi8-r', 'replace')
            #if style[0] > line_index:
            #    break
        #print sl
        self.styles = sl

        #bms = fb.bookmarks
        for i in fb.titles:
            self.lines[i][0] = TYPE_TITLE

        for i in fb.binaries:
            self.lines[i[3]-1] = [TYPE_IMAGE, i[2]]

            #print self.lines[i[1]][1].encode('koi8-r', 'replace'), i[1]

##         # [[line_index, start, end, type], [...]]
##         self.styles = []
##         i = 0
##         for style in fb.styles:
##             start, length, type = style
##             end = start + length
##             ln = 0
##             for t, l in self.lines[ln:]:
##                 print start, end, ln, i
##                 if start <= i <= end:
##                     #e = end > len
##                     self.styles.append((ln, start-i, end-i, type))
##                 ln += 1
##                 i += len(l)+1
##                 if i > end:
##                     break

        #print self.styles

            

        self.len_text = reduce(lambda x, y: x+len(y[1]),
                               filter(lambda l: l[0] in (TYPE_TEXT, TYPE_TITLE),
                                      self.lines), 0)


        self.current_page_line_index = line_index
        self.current_page_char_index = char_index

        gtk.idle_add(self.emit_position_changed)



    def set_font(self, font):
        self.font = pango.FontDescription(font)
        pangolayout = self.create_pango_layout(' ')
        pangolayout.set_font_description(self.font)
        self.space_width = pangolayout.get_extents()[1][2]/PANGO_SCALE
        self.text_height = pangolayout.get_extents()[1][3]/PANGO_SCALE

        self.title_font = self.font.copy()
        self.title_font.set_weight(pango.WEIGHT_BOLD)

        #self.set_tab_width()


    def set_tab_width(self, tab_width=None):
        if tab_width is None:
            tab_width = self.tabs_width
        pl = self.create_pango_layout(' '*tab_width)
        width, height = pl.get_pixel_size()

        ta = pango.TabArray(1, TRUE)
        ta.set_tab(0, pango.TAB_LEFT, width)
        self.tab_array = ta
        self.tabs_width = tab_width


    def realize(self):
        gtk.DrawingArea.realize(self)
        self.gc = self._parent.style.fg_gc[gtk.STATE_NORMAL]
        self.pixbuf = gdk.pixbuf_new_from_file(self.skin_file)
        self.cur_pixbuf = self.pixbuf.scale_simple(self._width, self._height,
                                                   gdk.INTERP_BILINEAR)


    def area_expose_cb(self, area, event):
        x, y, w, h, d = self.window.get_geometry()
        if self.pixmap_area and self._width == w and self._height == h:
            self.window.draw_drawable(self.gc, self.pixmap_area,
                                      0, 0, 0, 0, w, h)
        else:
            self.draw_text()


    def draw_text(self):

        x, y, widget_width, widget_height, depth = self.window.get_geometry()

        if not self.pixmap_area or self._width != widget_width or self._height != widget_height:
            self.cur_pixbuf = self.pixbuf.scale_simple(widget_width,
                                                       widget_height,
                                                       gdk.INTERP_BILINEAR)
            self._width = widget_width
            self._height = widget_height

            self.pixmap_area = gdk.Pixmap(None,
                                          widget_width,
                                          widget_height,
                                          depth)
##             self.prev_pixmap_area = gdk.Pixmap(None,
##                                                widget_width,
##                                                widget_height,
##                                                depth)
##             self.next_pixmap_area = gdk.Pixmap(None,
##                                                widget_width,
##                                                widget_height,
##                                                depth)

        #if not self.pixmap_area:
        self._draw(self.pixmap_area,
                   self.current_page_line_index,
                   self.current_page_char_index)

        self.window.draw_drawable(self.gc, self.pixmap_area,
                                  0, 0, 0, 0, widget_width, widget_height)

        #gtk.idle_add(self._draw_fb)

##     def _draw_fb(self):
##         # draw fore page
##         if not self.next_pixmap_area:
##             self._draw(self.next_pixmap_area,
##                        self.next_page_line_index,
##                        self.next_page_char_index)
##         # draw back page
##         if not self.prev_pixmap_area:
##             line_index, index = self.calc_back()
##             self._draw(self.prev_pixmap_area, line_index, index)


    def _draw(self, area_window, line_index, index):

        widget_width = self._width
        widget_height = self._height

        self.cur_pixbuf.render_to_drawable(area_window, self.gc, 0, 0,
                                           0, 0,
                                           -1, -1,
                                           FALSE, 0, 0)

        text_area_height = widget_height-self.top_margin-self.bottom_margin

        # draw left page
        x_offset = self.left_page_left_margin
        y_offset = self.top_margin
        text_area_width = widget_width/2-self.left_page_left_margin \
                          -self.left_page_right_margin

        if index: # draw tail of line if need
            prev_index = index
            type, line = self.lines[line_index]
            #is_title = type==TYPE_TITLE and 1 or 0
            y_offset, index = self.draw_line(area_window,
                                             line_index,
                                             index,
                                             x_offset,
                                             y_offset,
                                             text_area_width,
                                             text_area_height)

            if not index:
                line_index += 1
            else:
                index += prev_index

        if y_offset+self.text_height <= self.top_margin+text_area_height:
            for type, line in self.lines[line_index:]:
                if type == TYPE_IMAGE:
                    #is_top = y_offset==self.top_margin and 1 or 0
                    yy = self.draw_image(area_window,
                                               line,
                                               x_offset,
                                               y_offset,
                                               text_area_width,
                                               text_area_height)
                    if yy >= 0:
                        line_index += 1
                        index = 0
                        y_offset = yy
                    else:
                        break
                    #print line
                else:
                    #is_title = type==TYPE_TITLE and 1 or 0
                    y_offset, index = self.draw_line(area_window,
                                                     line_index,
                                                     0,
                                                     x_offset,
                                                     y_offset,
                                                     text_area_width,
                                                     text_area_height)

                    if not index:
                        line_index += 1
                if y_offset+self.text_height > self.top_margin+text_area_height:
                    break

        # draw right page
        x_offset = widget_width/2+self.right_page_left_margin
        y_offset = self.top_margin
        text_area_width = widget_width/2-self.right_page_left_margin \
                          -self.right_page_right_margin

        if index: # draw tail of line if need
            prev_index = index
            type, line = self.lines[line_index]
            #is_title = type==TYPE_TITLE and 1 or 0
            y_offset, index = self.draw_line(area_window,
                                             line_index,
                                             index,
                                             x_offset,
                                             y_offset,
                                             text_area_width,
                                             text_area_height)

            if not index:
                line_index += 1
            else:
                index += prev_index

        if y_offset+self.text_height <= self.top_margin+text_area_height:
            for type, line in self.lines[line_index:]:
                if type == TYPE_IMAGE:
                    #print line
                    #is_top = y_offset==self.top_margin and 1 or 0
                    yy = self.draw_image(area_window,
                                    line,
                                    x_offset,
                                    y_offset,
                                    text_area_width,
                                    text_area_height)
                    if yy >= 0:
                        line_index += 1
                        index = 0
                        y_offset = yy
                    else:
                        break
                else:
                    #is_title = type==TYPE_TITLE and 1 or 0
                    y_offset, index = self.draw_line(area_window,
                                                     line_index,
                                                     0,
                                                     x_offset,
                                                     y_offset,
                                                     text_area_width,
                                                     text_area_height)


                    if not index:
                        line_index += 1
                if y_offset+self.text_height > self.top_margin+text_area_height:
                    break

        self.next_page_char_index = index
        self.next_page_line_index = line_index

        #area_window.thaw_updates()

        return


    def get_pixbuf(self, data, max_width): #, max_height):

        data = base64.decodestring(data)
        tmp_file = tempfile.mktemp()
        try:
            open(tmp_file, 'w').write(data)
            pb = gdk.pixbuf_new_from_file(tmp_file)
            #pb = gtk.gdk.pixbuf_new_from_inline(len(data), data, TRUE)
        finally:
            os.remove(tmp_file)
        w = pb.get_width()
        h = pb.get_height()
        if w > max_width:

            pb = pb.scale_simple(max_width,
                                 h*float(max_width)/w,
                                 gdk.INTERP_BILINEAR)
        return pb


    def draw_image(self, area_window,
                   data,
                   x_offset,
                   y_offset,
                   text_area_width,
                   text_area_height):

##         data = base64.decodestring(data)
##         tmp_file = tempfile.mktemp()
##         try:
##             open(tmp_file, 'w').write(data)
##             pb = gdk.pixbuf_new_from_file(tmp_file)
##             #pb = gtk.gdk.pixbuf_new_from_inline(len(data), data, TRUE)
##             #self.binary_image.set_from_pixbuf(pb)
##         finally:
##             os.remove(tmp_file)
        pb = self.get_pixbuf(data, text_area_width)
        is_top = y_offset==self.top_margin and 1 or 0

        width = pb.get_width()
        height = pb.get_height()
        #print height
        if not is_top and y_offset+height > text_area_height:
            return -1
        else:

            #pb = pb.add_alpha(TRUE, chr(255), chr(255), chr(255)) # 
            pb.render_to_drawable(area_window, self.gc, 0, 0,
                                  x_offset+(text_area_width-width)/2, y_offset,
                                  width, height,
                                  FALSE, 0, 0)




##             render_to_drawable_alpha(drawable,
##                                      src_x, src_y,
##                                      dest_x, dest_y,
##                                      width, height,
##                                      alpha_mode, alpha_threshold,
##                                      dither, x_dither, y_dither)

##             pb.render_to_drawable_alpha(area_window, 0, 0,
##                                         x_offset+(text_area_width-width)/2,
##                                         y_offset,
##                                         width, height,
##                                         0, 0,
##                                         FALSE, 0, 0)



            return y_offset+height



    def draw_line(self, area_window,
                  line_index,
                  char_index,
                  x_offset, y_offset,
                  text_area_width, text_area_height):



        pangolayouts_list = self.get_pangolayout_list(text_area_width,
                                                      line_index,
                                                      char_index)


        if not pangolayouts_list:
            # empty line
            return y_offset+self.text_height, 0


        index = 0
        y = y_offset
        pli = 0
        for pl in pangolayouts_list:
            if y+self.text_height > text_area_height+self.top_margin:
                break
            pli += 1
            text_width = reduce(lambda a, b: a+b, map(lambda x: x[1], pl), 0)

            is_title = self.lines[line_index][0]==TYPE_TITLE and 1 or 0

            if is_title:
                space = self.space_width
            elif len(pl) <= 1:
                # single word
                space = 0
            elif pli == len(pangolayouts_list):
                # last line in this paragraph
                space = self.space_width
            else:
                space = (text_area_width-text_width)/(len(pl)-1)


            if is_title:
                p_length = 0
                for p in pl:
                    p_length += p[1]
                x = x_offset + (text_area_width - (len(pl)-1)*space - p_length)/2
            else:
                x = x_offset


##             sl = []
##             for style in self.styles:
##                 #print style[0], line_index
##                 if style[0] == line_index-1:
##                     sl.append(style)
##                     print self.lines[line_index][1].encode('koi8-r', 'replace')
##                 #if style[0] > line_index:
##                 #    break
##             print sl


            for i in range(len(pl)):
                index = pl[i][2]

                if space and i == len(pl)-1 \
                       and not pli == len(pangolayouts_list) and not is_title:
                    # last word in this line
                    x = x_offset+text_area_width-pl[i][1]
                if pli == len(pangolayouts_list):
                    index = 0

##                 for st in sl:
##                     start = st[1]
##                     end = st[1]+st[2]
##                     print start, '=', end, len(line)
##                     print '++', line.encode('koi8-r', 'replace')
##                     print '-', line[start:end].encode('koi8-r', 'replace')
##                     if start <= index:
##                         s = start<=prev_index and prev_index or start
##                         e = end<=index and end or index
##                         print '>', s, e
                        


                area_window.draw_layout(self.gc, x, y, pl[i][0],
                                        foreground=self.color)

                x += pl[i][1]+space

            y += self.text_height

        return y, index


    def get_pangolayout_list(self, text_area_width,
                             line_index,
                             char_index=0,
                             end_char_index=0):


        '''create list:
        [
          [[pangolayout, width, index] [pangolayout, width, index] [...]]
          [[pangolayout, width, index] [pangolayout, width, index] [...]]
        ]
        '''

        if end_char_index:
            line = self.lines[line_index][1][char_index:end_char_index]
        else:
            line = self.lines[line_index][1][char_index:]

        if self.lines[line_index][0] == TYPE_TITLE:
            hyphanate = 0
            font = self.title_font
        else:
            hyphanate = 1
            font = self.font

        words_list = []
        word = ''
        index = 0
        prev_index = 0
        for c in line:
            index += 1
            if c in string.whitespace:
                if word:
                    words_list.append((word, prev_index, index))
                    prev_index = index
                    word = ''
                    continue
            word += c
        if word: words_list.append((word, prev_index, index+1))


        text_width = 0
        total_width = 0
        line_number = 0
        pangolayout_list = []  # line
        pangolayouts_list = [] # paragraph
        index = 0

        styles = self.styles.get(line_index, [])


        for word, prev_index, index in words_list:
            #print '>', prev_index, index, word.encode('koi8-r', 'replace')

##             for start, end, style in styles:
##                 if start <= index and end >= prev_index:
##                     #print words_list
##                     print start, end, prev_index, index, word.encode('koi8-r', 'replace')
##                     print self.lines[line_index][1][start:end].encode('koi8-r', 'replace')


            pangolayout = self.create_pango_layout(word.encode('utf-8'))

            pangolayout.set_font_description(font)
            pangolayout.set_tabs(self.tab_array)

            width = pangolayout.get_extents()[1][2]/PANGO_SCALE

            if total_width+width > text_area_width:

                if hyphanate:
                    ## hyphanation
                    wl = hyphenate(word)
                    for w in wl:
                        pl = self.create_pango_layout(w.encode('utf-8')+'-')
                        pl.set_font_description(font)

                        wd = pl.get_extents()[1][2]/PANGO_SCALE
                        if total_width+wd < text_area_width:
                            index -= len(word)-len(w)+1
                            #?pangolayout_list.append((pl, wd, index))
                            text_width += wd
                            # skip leading '-'
                            if word[len(w)] == '-':
                                wt = word[len(w)+1:]
                                index += 1
                            else:
                                wt = word[len(w):]
                            pangolayout = self.create_pango_layout(wt.encode('utf-8'))
                            pangolayout.set_font_description(font)
                            pangolayout.set_tabs(self.tab_array)

                            width = pangolayout.get_extents()[1][2]/PANGO_SCALE
                            pangolayout_list.append((pl, wd, index))
                            break

                if pangolayout_list:
                    pangolayouts_list.append(pangolayout_list)

                line_number += 1
                text_width = 0
                total_width = 0
                pangolayout_list = []

            text_width += width
            total_width += width+self.space_width
            pangolayout_list.append((pangolayout, width, index))

        if pangolayout_list:
            pangolayouts_list.append(pangolayout_list)

        return pangolayouts_list


    def calc_back(self):

        def calc_back_1(line_index, index, text_area_width, text_area_height):
            new_line_index = line_index
            new_index = index
            y = 0
            if index:
                # tail of line

                pangolayouts_list = self.get_pangolayout_list(text_area_width,
                                                              line_index,
                                                              0,
                                                              index)
                pangolayouts_list.reverse()

                if not pangolayouts_list:
                    # empty line, not needed?
                    print 'empty line 1', new_line_index
                    y += self.text_height
                    if y+self.text_height > text_area_height:
                        return new_line_index-1, 0

                for pli in range(len(pangolayouts_list)):

                    y += self.text_height

                    if y+self.text_height > text_area_height:
                        if pli < len(pangolayouts_list)-1:
                            new_index = pangolayouts_list[pli+1][-1][2]
                        else:
                            new_index = 0

                        return new_line_index, new_index

##             if y+self.text_height > text_area_height:
##                 # not needed?
##                 print 'X'*24
##                 return new_line_index, 0

            lines = self.lines[:new_line_index]
            lines.reverse()

            for type, line in lines:

                new_line_index -= 1

                if type == TYPE_IMAGE:
                    y += self.text_height

                    pb = self.get_pixbuf(line, text_area_width)
                    h = pb.get_height()

                    if y+h >= text_area_height:
                        return new_line_index, 0
                    y += h #+self.text_height
                    continue

                pangolayouts_list = self.get_pangolayout_list(text_area_width,
                                                              new_line_index)

                pangolayouts_list.reverse()


                if not pangolayouts_list:
                    # empty line
                    y += self.text_height
                    if y+self.text_height > text_area_height:
                        return new_line_index, 0
                    continue

                for pli in range(len(pangolayouts_list)):

                    y += self.text_height

                    if y+self.text_height > text_area_height:

                        if pli < len(pangolayouts_list)-1:
                            new_index = pangolayouts_list[pli+1][-1][2]
                        else:
                            new_index = 0

                        return new_line_index, new_index


            return 0, 0

        text_area_height = self._height-self.top_margin-self.bottom_margin

        line_index = self.current_page_line_index
        index = self.current_page_char_index

        # right page
        x_offset = self._width/2+self.right_page_left_margin
        y_offset = self.top_margin
        text_area_width = self._width/2-self.right_page_left_margin \
                          -self.right_page_right_margin

        #print 'right >>', text_area_width
        line_index, index = calc_back_1(line_index, index,
                                        text_area_width, text_area_height)

        #print '###', line_index, index
        # left page
        x_offset = self.left_page_left_margin
        y_offset = self.top_margin
        text_area_width = self._width/2-self.left_page_left_margin \
                          -self.left_page_right_margin

        #print 'left >>', text_area_width
        line_index, index = calc_back_1(line_index, index,
                                        text_area_width, text_area_height)

        #print '###', line_index, index
        return line_index, index

##     def split_line(self, line):
##         words_list = []
##         word = ''
##         index = 0
##         prev_index = 0
##         for c in line:
##             index += 1
##             if c in string.whitespace:
##                 if word:
##                     words_list.append((word, prev_index))
##                     prev_index = index
##                     word = ''
##                     continue
##             word += c
##         if word: words_list.append((word, prev_index))
##         return words_list

    def go_fore_page(self):
        if self.next_page_line_index == len(self.lines):
            return
        #print '==go_fore_page==', self.current_page_line_index, self.current_page_char_index
        self.current_page_line_index = self.next_page_line_index
        self.current_page_char_index = self.next_page_char_index
        self.draw_text()
        gtk.idle_add(self.emit_position_changed)


    def go_back_page(self):
        if self.current_page_line_index == 0 \
           and self.current_page_char_index == 0:
            return
        line_index, index = self.calc_back()
        #print '==go_back_page==', line_index, index
        self.current_page_line_index = line_index
        self.current_page_char_index = index
        self.draw_text()
        gtk.idle_add(self.emit_position_changed)





    def emit_position_changed(self):
        pos = float(reduce(lambda x, y: x+len(y[1]),
                           filter(lambda l: l[0] in (TYPE_TEXT, TYPE_TITLE),
                                  self.lines[:self.current_page_line_index]),
                           0) \
                    + self.current_page_char_index) / len(self.text)
        self.emit('position-changed', pos)


    def set_position(self, pos):
        index = len(self.text)*pos
        i = 0
        n = 0
        for type, line in self.lines:
            if index <= n:
                break
            n += len(line)+1
            i += 1
        self.current_page_line_index = i
        self.current_page_char_index = 0
        self.draw_text()


    def key_press_event_cb(self, w, event):
        print 'key_press_event_cb'

    def key_release_event_cb(self, w, event):
        #print 'key_release_event_cb'
        if event.keyval in self.keysyms_next:
            self.go_fore_page()
            return TRUE
        if event.keyval in self.keysyms_prev:
            self.go_back_page()
            return TRUE

    def button_release_event_cb(self, w, event):
        if event.button == 1:
            self.go_fore_page()
        elif event.button == 2:
            #print 'event.button == 2'
            self.go_back_page()


##----------------------------------------------------------------------

vowels = unicode('ţ', 'koi8-r')
consonants = unicode('', 'koi8-r')
hardsoftsigns = unicode('', 'koi8-r')

def hyphenate(word):
    # based on code by Mike Matsnev
    # ("Haali Reader" http://haali.cs.msu.ru/pocketpc/)

    def hyphenate_word(word):
        # create list of hyphenation
        # i.e. "" -> ['', '', '']

        length = len(word)

        i = 0
        wl = []
        w = ''
        while i < length:
            if word[i] in vowels:
                for j in range(i+1, length):
                    if word [j] in vowels:
                        if word[i+1] in consonants \
                               and word[i+2] in consonants:
                            w += word[i]
                            i += 1
                        elif word[i+1] in consonants \
                                 and word[i+2] in hardsoftsigns:
                            w += word[i:i+2]
                            i += 2
                        if 1 <= i < length-2: # and i >= 1:
                            wl.append(w+word[i])
                        break

            w += word[i]
            i += 1

        return wl

    words_list = []
    w = u''
    ww = u''

    # split words
    for i in word:
        if i.isalnum():
            ww += i
        else:
            if ww:
                for j in hyphenate_word(ww):
                    words_list.append(w+j)
            if i == '-':
                words_list.append(w+ww)
            w += ww+i
            ww = u''

    if ww:
        for j in hyphenate_word(ww):
            words_list.append(w+j)

    words_list.reverse()
    return words_list



if __name__ == '__main__':
    import sys
    if len(sys.argv) != 2:
        sys.exit('usage: %s file' % sys.argv[0])

    window = gtk.Window()
    window.connect('destroy', lambda w: gtk.main_quit())
    ob = OrnamentBook(window)
    window.add(ob)

    #text = ''
    #for i in xrange(20):
    #    text = text+str(i)*3+' '
    #text = (text+' aa bb cc ddddd\n')*7
    text = open(sys.argv[1]).read()
    ob.set_text(unicode(text, 'utf-8'))
    ob.realize()
    window.show_all()

    gtk.main()
